/*
 * Interactive tool for exploring the topic of applying convolutions as image filters.
 *
 * Author: Michael Goldwasser
 */

float[][] kernel;
PImage original, edited;
boolean showEdited = true;
int reset = 0;
java.awt.Insets insets;

int offset = 0;  // used for emboss filters

int panelWidth = 400;
int panelHeight = 400;

void setup() {
  size(400, 400);
  textSize(20);
  textAlign(CENTER, CENTER);
  noStroke();

  frame.pack();
  insets = frame.getInsets();
  loadNewImage("data/image_basketball.jpg");  // default image
  loadKernel("data/kernel_identity3.txt"); // default kernel
  showEdited = false;   // given that its the identity
  displayImage();
}

// invokes user dialog for selecting an image file
void pickImage() {
  selectInput("Select image file", "pickImageHandler", new File("images"));
}

// handler for image-selection dialog
void pickImageHandler(File f) {
  if (f != null) {
    loadNewImage(f.getAbsolutePath());
  }
}

// used to load a given image file and resize the canvas appropriately
void loadNewImage(String filename) {
  original = loadImage(filename);
  if (kernel != null) {
    edited = convolution(original, kernel, 0);
  }

  int ww = max(MIN_WINDOW_WIDTH, panelWidth + original.width) + insets.left + insets.right;
  int wh = Math.max(MIN_WINDOW_HEIGHT, Math.max(panelHeight, original.height)) + insets.top + insets.bottom;
  frame.setSize(ww, wh);
  size(panelWidth + original.width, Math.max(original.height,panelHeight));
  background(200);
  reset = 5;
}

// invokes user diaolog for selecting a kernel file
void pickKernel() {
  selectInput("Select convolution kernel file", "pickKernelHandler", new File("kernels"));
}

// handler for kernel-selection dialog
void pickKernelHandler(File f) {
  if (f != null) {
    loadKernel(f.getAbsolutePath());
  }
}

// loads and applies a selected kernel from file
void loadKernel(String filename) {
  boolean fail = false;
  String lines[] = loadStrings(filename);
  int rows = 0;
  int cols = 0;
  for (int j=0; j < lines.length; j++) {
    String[] temp = splitTokens(lines[j], " ");
    if (temp.length > 0) {
      rows++;
      if (cols == 0) {
        cols = temp.length;
      } else if (temp.length != cols) {
        fail = true;
        break;
      }
    }
  }

  if (!fail) {
    kernel = new float[rows][cols];
    for (int j=0; j < rows; j++) {
      float[] temp = float(splitTokens(lines[j], " "));
      for (int k=0; k < cols; k++) {
        kernel[j][k] = temp[k];
      }
    }
    offset = (filename.contains("emboss") ? 127 : 0);   // somewhat of a hack
    applyKernel();
  } else {
    println("Illegal kernel file");
  }
}

// apply current kernel to original image
void applyKernel() {
  edited = convolution(original, kernel, offset);
  showEdited = true;
  displayAll();
}

void draw() {
  if (reset > 0) {         // hack for now...
    reset--;
    if (reset == 0) {
      background(200);
      displayAll();
    }
  }
}

void displayImage() {
  if (original != null) {
    if (edited != null && showEdited) {
      image(edited, panelWidth, 0);
  //    frame.setTitle("Filtered");
    } else {
      image(original, panelWidth, 0);
   //   frame.setTitle("Original");
    }
  }
}

void displayKernel() {
  if (kernel != null) {
    fill(200); // background color 
    rect(0, 0, panelWidth, panelHeight);

    fill(0);
    float hGap = panelWidth / max(9, kernel[0].length);
    float vGap = panelHeight / (1 + max(9, kernel.length));
    float hOffset = (9 - min(9, kernel[0].length))/2;
    for (int y=0; y < kernel.length; y++) {
      for (int x=0; x < kernel[y].length; x++) {
        text(""+nfs(int(kernel[x][y]), 1), (0.5+x+hOffset)*hGap, (1.5+y)*vGap);
      }
    }
  }
}

void displayAll() {
  displayImage();
  displayKernel();
}

void mouseClicked() {
  showEdited = !showEdited;  // toggle setting
  displayImage();
}

void keyPressed() {
  switch (key) {
  case 's':
    save("output.jpg");
    break;

  case 'r':
    showEdited = false;
    break;

  case 'i':
    pickImage();
    break;

  case 'k':
    pickKernel();
    break;
  }
}

